@@ -25,13 +25,12 @@
 #include "heif_limits.h"
 #include "nclx.h"
 
-#include <sstream>
 #include <iomanip>
 #include <utility>
 #include <iostream>
 #include <algorithm>
-#include <string.h>
-#include <assert.h>
+#include <cstring>
+#include <cassert>
 
 
 using namespace heif;
@@ -1639,7 +1638,7 @@ Error color_profile_nclx::parse(BitstreamRange& range)
 
 Error color_profile_nclx::get_nclx_color_profile(struct heif_color_profile_nclx** out_data) const
 {
-  *out_data = (struct heif_color_profile_nclx*) malloc(sizeof(struct heif_color_profile_nclx));
+  *out_data = alloc_nclx_color_profile();
 
   if (*out_data == nullptr) {
     return Error(heif_error_Memory_allocation_error,
@@ -1671,6 +1670,22 @@ Error color_profile_nclx::get_nclx_color_profile(struct heif_color_profile_nclx*
 }
 
 
+struct heif_color_profile_nclx* color_profile_nclx::alloc_nclx_color_profile()
+{
+  auto profile = (heif_color_profile_nclx*) malloc(sizeof(struct heif_color_profile_nclx));
+
+  if (profile) {
+    profile->version = 1;
+    profile->color_primaries = heif_color_primaries_unspecified;
+    profile->transfer_characteristics = heif_transfer_characteristic_unspecified;
+    profile->matrix_coefficients = heif_matrix_coefficients_ITU_R_BT_601_6;
+    profile->full_range_flag = true;
+  }
+
+  return profile;
+}
+
+
 void color_profile_nclx::free_nclx_color_profile(struct heif_color_profile_nclx* profile)
 {
   free(profile);
@@ -1686,6 +1701,15 @@ void color_profile_nclx::set_default()
 }
 
 
+void color_profile_nclx::set_undefined()
+{
+  m_colour_primaries = 2;
+  m_transfer_characteristics = 2;
+  m_matrix_coefficients = 2;
+  m_full_range_flag = true;
+}
+
+
 void color_profile_nclx::set_from_heif_color_profile_nclx(const struct heif_color_profile_nclx* nclx)
 {
   m_colour_primaries = nclx->color_primaries;
