@@ -1006,7 +1006,7 @@ Error HeifContext::decode_image_planar(heif_item_id ID,
     // (non-NCLX) profile later.
     auto nclx = std::dynamic_pointer_cast<const color_profile_nclx>(imginfo->get_color_profile());
     if (nclx) {
-      img->set_color_profile(nclx);
+      img->set_color_profile_nclx(nclx);
     }
 
     heif_colorspace target_colorspace = (out_colorspace == heif_colorspace_undefined ?
@@ -1030,8 +1030,10 @@ Error HeifContext::decode_image_planar(heif_item_id ID,
       }
     }
 
-    auto colorProfileInMetadata = imginfo->get_color_profile();
-    img->set_color_profile(colorProfileInMetadata);
+    auto colorProfileInMetadata = std::dynamic_pointer_cast<const color_profile_nclx>(imginfo->get_color_profile());
+    if (colorProfileInMetadata) {
+      img->set_color_profile_nclx(colorProfileInMetadata);
+    }
   }
   else if (image_type == "grid") {
     std::vector<uint8_t> data;
@@ -1179,7 +1181,7 @@ Error HeifContext::decode_image_planar(heif_item_id ID,
     }
   }
 
-  
+
   return Error::Ok;
 }
 
@@ -1617,12 +1619,17 @@ create_alpha_image_from_image_alpha_channel(const std::shared_ptr<HeifPixelImage
                       heif_colorspace_YCbCr, heif_chroma_420);
   alpha_image->copy_new_plane_from(image, heif_channel_Alpha, heif_channel_Y);
 
-  int bpp = image->get_bits_per_pixel(heif_channel_Alpha);
-  int half_range = 1<<(bpp-1);
+  uint8_t bpp = image->get_bits_per_pixel(heif_channel_Alpha);
+  uint16_t half_range = static_cast<uint16_t>(1 << (bpp - 1));
 
   alpha_image->fill_new_plane(heif_channel_Cb, half_range, chroma_width, chroma_height, bpp);
   alpha_image->fill_new_plane(heif_channel_Cr, half_range, chroma_width, chroma_height, bpp);
 
+  auto nclx = std::make_shared<color_profile_nclx>();
+  nclx->set_undefined();
+  nclx->set_full_range_flag(true); // in default, but just to be sure in case defaults change
+  alpha_image->set_color_profile_nclx(nclx);
+
   return alpha_image;
 }
 
@@ -1791,13 +1798,10 @@ Error HeifContext::Image::encode_image_as_hevc(std::shared_ptr<HeifPixelImage> i
 
   heif_colorspace colorspace = image->get_colorspace();
   heif_chroma chroma = image->get_chroma_format();
-  auto color_profile = image->get_color_profile();
-  if (!color_profile) {
-    auto nclx = std::make_shared<color_profile_nclx>();
-    nclx->set_default();
-    color_profile = nclx;
+  auto nclx_profile = image->get_color_profile_nclx();
+  if (!nclx_profile) {
+    nclx_profile = std::make_shared<color_profile_nclx>();
   }
-  auto nclx_profile = std::dynamic_pointer_cast<const color_profile_nclx>(color_profile);
 
   if (encoder->plugin->plugin_api_version >= 2) {
     encoder->plugin->query_input_colorspace2(encoder->encoder, &colorspace, &chroma);
@@ -1819,11 +1823,19 @@ Error HeifContext::Image::encode_image_as_hevc(std::shared_ptr<HeifPixelImage> i
   m_width = image->get_width(heif_channel_Y);
   m_height = image->get_height(heif_channel_Y);
 
-  if (color_profile &&
-      (input_class == heif_image_input_class_normal || input_class == heif_image_input_class_thumbnail)) {
-    m_heif_context->m_heif_file->set_color_profile(m_id, color_profile);
+  // --- choose which color profile to put into 'colr' box
+
+  if (input_class == heif_image_input_class_normal || input_class == heif_image_input_class_thumbnail) {
+    auto icc_profile = image->get_color_profile_icc();
+    if (icc_profile) {
+      m_heif_context->m_heif_file->set_color_profile(m_id, icc_profile);
+    }
+    else if (nclx_profile) {
+      m_heif_context->m_heif_file->set_color_profile(m_id, nclx_profile);
+    }
   }
 
+
   // --- if there is an alpha channel, add it as an additional image
 
   if (options->save_alpha_channel && image->has_channel(heif_channel_Alpha)) {
@@ -1924,11 +1936,9 @@ Error HeifContext::Image::encode_image_as_av1(std::shared_ptr<HeifPixelImage> im
 
   heif_colorspace colorspace = image->get_colorspace();
   heif_chroma chroma = image->get_chroma_format();
-  auto color_profile = image->get_color_profile();
+  auto color_profile = image->get_color_profile_nclx();
   if (!color_profile) {
-    auto nclx = std::make_shared<color_profile_nclx>();
-    nclx->set_default();
-    color_profile = nclx;
+    color_profile = std::make_shared<color_profile_nclx>();
   }
   auto nclx_profile = std::dynamic_pointer_cast<const color_profile_nclx>(color_profile);
 
@@ -1952,9 +1962,16 @@ Error HeifContext::Image::encode_image_as_av1(std::shared_ptr<HeifPixelImage> im
   m_width = image->get_width(heif_channel_Y);
   m_height = image->get_height(heif_channel_Y);
 
-  if (color_profile &&
-      (input_class == heif_image_input_class_normal || input_class == heif_image_input_class_thumbnail)) {
-    m_heif_context->m_heif_file->set_color_profile(m_id, color_profile);
+  // --- choose which color profile to put into 'colr' box
+
+  if (input_class == heif_image_input_class_normal || input_class == heif_image_input_class_thumbnail) {
+    auto icc_profile = image->get_color_profile_icc();
+    if (icc_profile) {
+      m_heif_context->m_heif_file->set_color_profile(m_id, icc_profile);
+    }
+    else if (nclx_profile) {
+      m_heif_context->m_heif_file->set_color_profile(m_id, nclx_profile);
+    }
   }
 
 
@@ -2021,6 +2038,21 @@ Error HeifContext::Image::encode_image_as_av1(std::shared_ptr<HeifPixelImage> im
   height = image->get_height();
   m_heif_context->m_heif_file->add_ispe_property(m_id, width, height);
 
+
+  if (encoder->plugin->plugin_api_version >= 3) {
+    uint32_t encoded_width, encoded_height;
+
+    encoder->plugin->query_encoded_size(encoder->encoder,
+                                        m_width, m_height,
+                                        &encoded_width,
+                                        &encoded_height);
+    if (m_width != encoded_width ||
+        m_height != encoded_height) {
+      m_heif_context->m_heif_file->add_clap_property(m_id, m_width, m_height,
+                                                     encoded_width, encoded_height);;
+    }
+  }
+
   return Error::Ok;
 }
 
