@@ -27,13 +27,10 @@
 #include "config.h"
 #endif
 
-#include <math.h>
-#include <memory>
-#include <string>
-#include <string.h>
-#include <stdio.h>
-#include <assert.h>
 #include <vector>
+#include <memory>
+#include <cstring>
+#include <cassert>
 
 #include <iostream>  // TODO: remove me
 
@@ -59,6 +56,7 @@ struct encoder_struct_rav1e
   bool data_read = false;
 };
 
+static const char* kError_out_of_memory = "Out of memory";
 
 static const char* kParam_min_q = "min-q";
 static const char* kParam_threads = "threads";
@@ -66,7 +64,7 @@ static const char* kParam_speed = "speed";
 
 static const char* kParam_chroma = "chroma";
 static const char* const kParam_chroma_valid_values[] = {
-    "420", "422", "444"
+    "420", "422", "444", nullptr
 };
 
 static int valid_tile_num_values[] = {1, 2, 4, 8, 16, 32, 64};
@@ -478,11 +476,39 @@ void rav1e_query_input_colorspace2(void* encoder_raw, heif_colorspace* colorspac
 }
 
 
+void rav1e_query_encoded_size(void* encoder, uint32_t input_width, uint32_t input_height,
+                              uint32_t* encoded_width, uint32_t* encoded_height)
+{
+  *encoded_width = std::max(input_width, 16U);
+  *encoded_height = std::max(input_height, 16U);
+}
+
+
 struct heif_error rav1e_encode_image(void* encoder_raw, const struct heif_image* image,
                                      heif_image_input_class input_class)
 {
   auto* encoder = (struct encoder_struct_rav1e*) encoder_raw;
 
+  // --- round image size to minimum size
+
+  uint32_t rounded_width, rounded_height;
+  rav1e_query_encoded_size(encoder,
+                           image->image->get_width(),
+                           image->image->get_height(),
+                           &rounded_width,
+                           &rounded_height);
+
+  bool success = image->image->extend_to_size(rounded_width, rounded_height);
+  if (!success) {
+    struct heif_error err = {
+        heif_error_Memory_allocation_error,
+        heif_suberror_Unspecified,
+        kError_out_of_memory
+    };
+    return err;
+  }
+
+
   const heif_chroma chroma = heif_image_get_chroma_format(image);
 
   uint8_t yShift = 0;
@@ -509,6 +535,11 @@ struct heif_error rav1e_encode_image(void* encoder_raw, const struct heif_image*
   }
 
   rav1eRange = RA_PIXEL_RANGE_FULL;
+  auto nclx = image->image->get_color_profile_nclx();
+  if (nclx) {
+    rav1eRange = nclx->get_full_range_flag() ? RA_PIXEL_RANGE_FULL : RA_PIXEL_RANGE_LIMITED;
+  }
+
   int bitDepth = image->image->get_bits_per_pixel(heif_channel_Y);
 
 
@@ -533,6 +564,16 @@ struct heif_error rav1e_encode_image(void* encoder_raw, const struct heif_image*
     return heif_error_codec_library_error;
   }
 
+  if (nclx &&
+      (input_class == heif_image_input_class_normal ||
+       input_class == heif_image_input_class_thumbnail)) {
+    if (rav1e_config_set_color_description(rav1eConfig.get(),
+                                           (RaMatrixCoefficients) nclx->get_matrix_coefficients(),
+                                           (RaColorPrimaries) nclx->get_colour_primaries(),
+                                           (RaTransferCharacteristics) nclx->get_transfer_characteristics()) == -1) {
+      return heif_error_codec_library_error;
+    }
+  }
 
   if (rav1e_config_parse_int(rav1eConfig.get(), "min_quantizer", encoder->min_q) == -1) {
     return heif_error_codec_library_error;
@@ -560,7 +601,7 @@ struct heif_error rav1e_encode_image(void* encoder_raw, const struct heif_image*
     }
   }
 
-  auto colorProfile = image->image->get_color_profile();
+  auto colorProfile = image->image->get_color_profile_nclx();
   if (auto nclxProfile = std::dynamic_pointer_cast<const heif_color_profile_nclx>(colorProfile)) {
     rav1e_config_set_color_description(rav1eConfig.get(),
                                        (RaMatrixCoefficients) nclxProfile->matrix_coefficients,
@@ -650,7 +691,7 @@ struct heif_error rav1e_get_compressed_data(void* encoder_raw, uint8_t** data, i
 
 static const struct heif_encoder_plugin encoder_plugin_rav1e
     {
-        /* plugin_api_version */ 2,
+        /* plugin_api_version */ 3,
         /* compression_format */ heif_compression_AV1,
         /* id_name */ "rav1e",
         /* priority */ RAV1E_PLUGIN_PRIORITY,
@@ -677,7 +718,8 @@ static const struct heif_encoder_plugin encoder_plugin_rav1e
         /* query_input_colorspace */ rav1e_query_input_colorspace,
         /* encode_image */ rav1e_encode_image,
         /* get_compressed_data */ rav1e_get_compressed_data,
-        /* query_input_colorspace (v2) */ rav1e_query_input_colorspace2
+        /* query_input_colorspace (v2) */ rav1e_query_input_colorspace2,
+        /* query_encoded_size (v3) */ rav1e_query_encoded_size
     };
 
 const struct heif_encoder_plugin* get_encoder_plugin_rav1e()
